import re
from urllib.parse import urlparse

from lxml.html import fromstring
from unidecode import unidecode

def hemsida(kommun):
    '''
    >>> hemsida('Vetlanda kommun')
    'http://vetlanda.se'
    '''
    m = re.match(r'^(.+[^s])(s?) kommun$', kommun)
    if not m:
        assert False, kommun
    w = unidecode(m.group(1).lower())
    x = re.sub(r'[ -]', '', w)
    for scheme in ['http', 'https']:
        for prefix in ['', 'www.']:
            yield '%s://%s%s.se' % (scheme, prefix, x)
            if m.group(2) == 's':
                yield '%s://%s%ss.se' % (scheme, prefix, x)

def kommuner(response):
    html = fromstring(response.content)
    return filter(lambda text: text.endswith(' kommun'), html.xpath('//a/text()'))

def tracking(response):
    html = fromstring(response.content)
    html.make_links_absolute(response.url)

    o = urlparse(response.url)
    first_party = (o.scheme, o.netloc)

    for src in html.xpath('//script/@src'):
        p = urlparse(src)
        third_party = (p.scheme, p.netloc)
        if third_party != first_party:
            yield '%s://%s' % third_party



import sys
from concurrent.futures import ThreadPoolExecutor

from vlermv import cache
import requests

from . import parse

@cache('~/.gkebd', cache_exceptions = True)
def get(url, *args, **kwargs):
    return requests.get(url, *args, headers = { 'user-agent': 'https://pypi.python.org/pypi/gkebd'})

def gkebd():
    url = 'https://sv.wikipedia.org/wiki/Lista_%C3%B6ver_Sveriges_kommuner'
    with ThreadPoolExecutor(20) as e:
        for row in e.map(gkebd_one, parse.kommuner(get(url))):
            if row != None:
                yield row

def gkebd_one(kommun):
    for url in parse.hemsida(kommun):
        try:
            r = get(url, timeout = 10)
        except requests.exceptions.ConnectionError:
            pass
        except requests.exceptions.ReadTimeout:
            pass
        else:
            return kommun, parse.tracking(r)
    else:
         sys.stderr.write('Could not find a page for %s\n' % kommun)

def cli():
    import csv
    w = csv.writer(sys.stdout)
    w.writerow(('kommun', 'third.party'))
    for kommun, trackers in gkebd():
        for tracker in trackers:
            w.writerow((kommun, tracker))



from distutils.core import setup

setup(name='gkebd',
      author='Thomas Levine',
      author_email='_@thomaslevine.com',
      description='Look for (illegal) third-party trackers on Swedish government websites.',
      url='https://dada.pink/gkebd/',
      packages=['gkebd'],
      install_requires = [],
      version='0.0.3',
      license='AGPL',
      entry_points = {'console_scripts': ['gkebd = gkebd.main:cli']},
)
